local AppName, AppNS = ... -- Retrieve the Add-On name and namespace

-- Retrieve the keys of the given table t.
local function keys(t)
	local r = {}
	for k, v in pairs(t) do
		r[#r + 1] = k
	end
	return r
end

-- Create a local variable containing all available keys
local Keys = keys(AppNS)

-- Register this addon with WowAce
local Wiki = LibStub("AceAddon-3.0"):NewAddon(AppName)
local L = LibStub("AceLocale-3.0"):GetLocale(AppName)

-- Setter for the Wiki source variable
function Wiki:SetSource(info, input)
	self.db.global.source = input
end

-- Getter for the Wiki source variable
function Wiki:GetSource(info)
	return self.db.global.source
end

-- Retrieve string parts from text
local function ParseCommand(text)
	if text:find("/") ~= 1 then
		return nil, text
	end
	
	local space = text:find(" ")
	if space then
		return text:sub(1, space - 1), text:sub(space + 1)
	end
	return text, nil
end

-- Retrieve the first item from a table t
local function first(t)
	for k, v in pairs(t) do
		return v
	end
	return nil
end

-- Initialization
function Wiki:OnInitialize()
	-- Register saved variables. Initialize with first key
	self.db = LibStub("AceDB-3.0"):New("WikiDB", {
		global = {
			source = first(Keys)
		}
	}, true)

	-- Create options
	LibStub("AceConfig-3.0"):RegisterOptionsTable(AppName, {
		name = AppName,
		handler = self,
		type = "group",
		args = {
			source = {
				type = "select",
				name = L["Database"],
				values = Keys,
				set = 'SetSource',
				get = 'GetSource'
			}
		}
	}, 'wiki') -- Register /wiki command here, since it is not registered by this addon otherwise (even though it is caught)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AppName)
	
	-- Slash command
	-- This is implemented somewhat weird, since we cannot keep the editbox open after the /wiki command without interrupting the OnEnterPressed event
	local EditBox = ChatEdit_ChooseBoxForSend()
	local EditBox_OnEnterPressed = EditBox:GetScript("OnEnterPressed")
	EditBox:SetScript("OnEnterPressed", function(this)
		local text = this:GetText()
		local command, args = ParseCommand(text)
		if command and command == "/wiki" then
			if args ~= nil then
				args = args:match('^%s*(.*%S)') or ''
			end
			local url = AppNS[Keys[self.db.global.source]]:GetUrl(args)
			this:SetText(url)
			this:HighlightText()
		else
			EditBox_OnEnterPressed(this)
		end
	end)
end
