local AppName, AppNS = ... -- Retrieve the Add-On name and namespace

AppNS = {}
Wiki_Source = {}

local byte, find, format, gsub = string.byte, string.find, string.format, string.gsub

local function split(str, sep)
   local fields = {}
   local pattern = format("([^%s]+)", sep)
   gsub(str, pattern, function(c) fields[#fields+1] = c end)
   return fields
end

function Wiki_Source:ParseHyperLink(itemLink)
	local _, _, Alpha, Color, DataString, ItemName = find(itemLink, "^|c(%x%x)(%x%x%x%x%x%x)|H([^%|]*)|h%[([^%[%]]*)%]|h|r$")
	return Alpha, Color, DataString, ItemName
end

function Wiki_Source:ParseData(dataString)
	local _, _, Ltype, ID, RemainingData = find(dataString, "^([^:]+):(%d+):(.*)$")
	-- TODO: More advanced parsing, i.e. bonus ids etc.
	return Ltype, ID, RemainingData
end

function Wiki_Source:EncodeURL(url)
	if url == nil then
		return
	end
	
	local char_to_hex = function(c)
		return format("%%%02X", byte(c))
	end
	
	url = gsub(url, "\n", "\r\n")
	url = gsub(url, "([^%w ])", char_to_hex)
	url = gsub(url, " ", "+")
	return url
end

function Wiki_Source:IsHyperlink(message)
	return self:ParseLink(message) ~= nil
end

function Wiki_Source:New()
	return setmetatable({}, {__index=self})
end
